require "test/unit"
require_relative "../lib/terse_ruby"

class TestExpansionRuby < Test::Unit::TestCase
        
    @@terse_file = "test_file_ruby.txt"
    @@expanded_file = "test_file_ruby.rb"
    @@premade_file = "expected_test_file_ruby.rb"
    @@argv_altered = false

    # not used - invoking via cmd means that the gem has to be rebuild & reinstalled after every code change, which does not lend itself to a speedy dev process
    # TODO add test that deliberately invokes via system
    @@cmd = "ruby -e \"require 'terse_ruby'; TerseRuby.scan_files ARGV\" #{@@terse_file}"

    def switch_dir
        # Change this so we know where the expaneded file will be created
        @previous_wd ||= Dir.getwd # Preserve the old wd so we can switch back to it after the test
        Dir.chdir File.expand_path(File.dirname(__FILE__))    
    end

    def run_cmd
        # system @@cmd # old mechanism; don't do this, call TerseRuby.scan_files ARGV instead

        # ARGV is preserved from one test_ to the next, so we should only alter it once
        unless @@argv_altered
            ARGV << @@terse_file
            @@argv_altered = true
        end
        TerseRuby.scan_files ARGV
    end

    # Switch to the same directory as this unit-test file, and check that the expected file is not present
    def setup
        switch_dir
        raise "terse_ruby unit-tests found a pre-existing #{@@expanded_file} in #{Dir.getwd}; unit-tests will clean up after themselves but will not alter pre-existing files" if File.file? @@expanded_file
        raise "terse_ruby unit-tests could not locate the starting test file #{@@terse_file} in #{Dir.getwd}" unless File.file? @@terse_file
    end

    # Tests that the test file is expanded with the expected name
    def test_expanded_name
        assert_false(File.file?(@@expanded_file), "Expanded test file #{@@expanded_file} already existed before running test")
        run_cmd
        assert(File.exists?(@@expanded_file), "terse_ruby did not expand #{@@terse_file} into a file with name #{@@expanded_file}")
        assert(File.file?(@@expanded_file), "terse_ruby did not expand #{@@terse_file} into a file with name #{@@expanded_file}")
    end

    # Tests that the test file is expanded into the expected form
    # Expand the file, then compare the two line-by-line
    def test_expansion
        run_cmd
        assert(File.file?(@@expanded_file), "terse_ruby did not expand #{@@terse_file} into a file with name #{@@expanded_file}")
        
        expected_lines = File.readlines(@@premade_file)
        actual_lines = File.readlines(@@expanded_file)
        
        assert_equal(expected_lines.size, actual_lines.size, "Expanded file did not have the expected number of lines")
        
        # As expected_lines and actual_lines are arrays, we can just use Array#==
        assert_equal(expected_lines.sort, actual_lines.sort, "Expanded file did not have the expected lines in the expected order")
        assert_equal(expected_lines, actual_lines, "Expanded file did not have the expected lines")
    end

    # Delete the expected file from the expected dir
    def teardown
        if File.file?(@@expanded_file)
            count = File.delete @@expanded_file
            raise "Teardown failed to delete expanded file #{@@expanded_file}" unless count == 1
            raise "Teardown failed to delete expanded file #{@@expanded_file}" if File.file?(@@expanded_file)
            puts "Teardown deleted expanded file #{@@expanded_file}"
        end
        Dir.chdir @previous_wd # restore the working directory to what it was previously
    end
    
end
